﻿#ifndef NEMOSDKADAPTOR_H
#define NEMOSDKADAPTOR_H

#include <string>
#include <vector>
#include <QObject>
#include <QDebug>
#include <QJsonDocument>
#include <QJsonObject>

#include <memory>

#include "NemoSDK.h"
#include "meeting/MeetingLayout.h"
#include "wayland/waylandcapture.h"

using namespace NemoSDK;

struct SDKVideoFrameWrapper
{
    std::shared_ptr<uint8_t> buffer;
    size_t bufLen;
    int width;
    int height;
    int sarWidth;
    int sarHeight;
    int rotation;
    bool flipping;
};

const QString SDKError_NoError = "XYSDK:979001";
const QString SDKError_WrongPassword = "XYSDK:973001";

class NemoSdkAdaptor: public QObject, public INemoSDKCallbacks
{
    Q_OBJECT

public:
    static NemoSdkAdaptor *getInstance();
    QString error_info(const QString& code);
    void init(const QString& host = "", const QString& extId = "", const QString& clientId = "",
              const QString& clientSecret = "");
    void shutdown();
    void login(const std::string& name, const std::string& pwd);
    void loginXYAccount(const std::string& extId, const std::string& countryCode, const std::string& account, const std::string& pwd);
    void loginExtToken(const std::string& extId, const std::string& token);
    void loginExtUserId(const std::string& extId, const std::string& displayName, const std::string& extUID, const std::string& authCode);
    void logout();
    void loginWithExtId(const std::string& extID, const std::string& extUserId, const std::string& displayName);
    void makeCall(const std::string& number, const std::string& meetingPwd, SDKCallMode mode, const std::string& displayName,
                  const QString& meetingId = "");
    void acceptCall();
    void rejectCall();
    void dropCall();
    void setChairman(const std::string& name);
    void setMicMute(bool mute);
    void setVideoMute(bool mute);
    void setAudioPeakMeterEnabled(bool enable);
    void startCamera();
    void stopCamera();
    void startContentSharing(const QString& monitor, const SDKContentInfo& contentInfo);
    void updateContentCaptureRegion(int x, int y, int width, int height);
    void stopContentSharing();
	void suspendContentShare();
	void restartContentShare();
    void log(const QString& info);
    std::vector<SDKMediaDevInfo> getDevList(SDKMediaDevType t);
    const XYArray<SDKAudioMeterInfo>& getAudioPeakMeters();
    const SDKStatistics& getStatistics();
    void chooseDev(SDKMediaDevType t, const char* devId);
    void setVideoProfile(const SDKVideoProfile& profile);
    void sendSpeakingRequest();
    void sendSpeakingEnd();
	void cancelSpeakingRequest() ;
    void setSpeakerVolume(int volume);
    int getSpeakerVolume();
    void startPlayWavFile(const char* wavfile, int loopTimes);
    void stopPlayWavfile();
    int getSpeakerPeakMeter();
    void enableListnerTest(bool enable);
    int getMicPeakMeter();
    void setMicVolume(int value);
	void startWhiteboard();
	void stopWhiteboard();
	void sendWhiteboardMessage(const char* msg);
    XYArray<SDKVideoStreamInfo> getLayout();
    void reqAutoLayout(int num, bool hasContent, const std::string& chairmanUri, MeetingLayout::LayoutMode layoutMode,
                       int pageIndex = 0, bool uiShowLocalWhenPageMode = true, const std::vector<QString>& calluirs = {});
    bool isCameraClosed()const;
    bool isVideoMuted()const;
    bool isConnected()const;
    bool isMicMuted()const;
    bool isContentSharing()const;
    bool isSdkClosed()const;
    bool isLogedIn()const;

    const XYArray<SDKMonitorInfo>& getMonitorList();
    void enableLocalFD(SDKFaceType faceType, bool enable);
    void quickLogin(const char* extID, const char* account);
    void getFaceInfo(const XYArray<int32_t>& faceId);
    void enableEletronicBadge(bool enable);
    void enableAudioDump();

    void setVirtualBgMode(SDKVirtualBgMode mode);
    void setVirtualBgImage(const char* imgPath);
    void setVideoBeautyEffect(SDKBeautyStyle style, int effectLevel);
    void setVideoFilterEffect(SDKFilterStyle style, int effectLevel);
    void logUpload(const std::string& comments);
    void getGPUInfo();
    void switchCallMode(bool audioMode, bool audioMute, bool videoMute);
    void enableAEC(bool enable);
    bool isRecordEnable() const;
    void setRecordEnable(bool enable);
    bool isRecordStartBySelf() const;
    void startRecord();
    void stopRecord();

    QString getConfMgmtUrl(const char* confNum);
    void releaseAudioCapSource();
    void enablePulseHelper(bool enable);

    void putContentData(void *argb, size_t bufLen, int width, int height, int stride);
	void setSubtitleLanguage(SDKLanguageType type);
    void checkAiCaptionSupport();
    void setProxy(ProxyType type, const ProxyNetAddress& proxy);
    void convertYUV2ARGB(const uint8_t* data, uint8_t* dst_argb, size_t bufLen, int width, int height, int rotate, bool flip);
    void setVideoFlip(bool filp);
    bool getVideoFlip() const;

signals:
    void sigOnVideoStreams(const std::vector<SDKVideoStreamInfo>& streams);
    void loginResult(bool loginOk);
    void confEpNumChange(int num);
	void sigMicEnergyReported(int energy);
    void sigKickOut(const QString& code);
    void sigLoginResult(SDKLoginState state, const QString& code, const SDKLoginInfo& info);
    void sigConnectionChanged(bool state);
    void sigConfStateChanged(const SDKCallStateInfo &callStateInfo);
    void sigCallInvited(const SDKCallInfo &callInfo);
    void sigConfMemberChanged(int num, bool hasContent, const QString& chairman);
    void sigConfMgmtChanged(const SDKConfMgmtInfo &confMgmtInfo);
    void sigRosterChanged(const SDKRosterInfo &info);
    void sigStreamChanged(const std::vector<SDKVideoStreamInfo>& streams);
    void sigVideoFrameReady(const QString& dataSourceId, const SDKVideoFrameWrapper& videoFrame);
    void sigNetworkIndicatorLevel(int level);
    void sigDeviceChanged(const SDKMediaDevType& devType);
    void sigVideoStateChanged(SDKVideoStatus status);
    void sigContentStateChanged(SDKContentState state);
	void sigRecvWhiteboardJson(const QString &jsonStr);
	void sigRecvWhiteboardFlag(bool state);
	void sigRecvWhiteboardCalluri(QString calluri);
    void sigConfHandupCancelled();
    void sigLoguploadResponse(const SDKRestApiResult& result);
    void sigConfHostChanged(bool isHost);
    void sigKConfCaptionChanged(const SDKConfCaption& confCaption);
    void sigAiCaptionInfoChanged(const SDKAiCaptionInfo& captionInfo);
    void sigSupportAiCaptionResult(bool isSupport);

private:
    NemoSdkAdaptor();
    virtual ~NemoSdkAdaptor();

    virtual void onKickOut(const XYString& code)override;
    virtual void onError(const XYString& code)override;
    virtual void onLoginStateChanged(SDKLoginState state, const XYString& code, const SDKLoginInfo& info)override;
    virtual void onCallStateChanged(const SDKCallStateInfo &callStateInfo)override;
    virtual void onCallInvited(const SDKCallInfo &callInfo)override;
    virtual void onHowlingDetected(bool detected)override;
    virtual void onNetworkStateChanged(bool connected)override;
    virtual void onVideoStatusChanged(SDKVideoStatus status)override;
    virtual void onVideoStreamChanged(const XYArray<SDKVideoStreamInfo>& streams)override;
    virtual void onContentStateChanged(SDKContentState state)override;
    virtual void onNetworkIndicatorLevel(int level)override;
    virtual void onDeviceChanged(const SDKMediaDevType& devType)override;
    virtual void onMicMuted(bool muted)override;
    virtual void onMicEnergyReported(int energy)override;
    virtual void onConfInfoChanged(const SDKConfInfo &info)override;
    virtual void onConfMgmtChanged(const SDKConfMgmtInfo & info)override;
    virtual void onRosterChanged(const SDKRosterInfo &info)override;
    virtual void onVideoFrameReady(const char* dataSourceId, const SDKVideoFrame& videoFrame)override;
    virtual void onWhiteboardMessages(const XYArray<XYString>& msgs);
    virtual void onWhiteboardStateChanged(const SDKWhiteboardState& state);
    virtual void onConfHandupCancelled()override;
    virtual void onGPUInfoResult(const XYArray<XYString> &gInfo) override;
    virtual void onAIFaceRecv(const SDKAIParam& info) override;
    virtual void onAIFaceInfo(const XYString& data) override;
    virtual void onRecordStatusNotification(int callIndex, bool isStart, const XYString& uri,
                                            const XYString& callUri, const XYString& status) override;
    virtual void onRecordingStateChanged(SDKRecordingState recordState, const XYString& reason,  bool isRecorded) override;
    virtual void onMeetingMuteQuery(const XYString& meetingId, const XYString& key) override;
    virtual void onLoguploadResult(const SDKRestApiResult& result) override;
    virtual void onConfHostChanged(const SDKConfHost& meetingHost) override;
    virtual void onInOutReminder(const XYArray<SDKInOutInfo>& info) override;
    virtual void onConfCaptionChanged(const SDKConfCaption& notify) override;
    virtual void onSupportAiCaptionResult(bool isSupport) override;
    virtual void onAiCaptionInfoChanged(const SDKAiCaptionInfo& info) override;

private:
    INemoSDK *_sdk = nullptr;
    QJsonObject jsonObj;
    bool videoMuted;
    bool _isRecordStartBySelf = false;
    bool micMuted;
    bool connected;
    bool cameraClosed = true;
    bool contentSharing;
    bool sdkClosed = true;
    QString calluri;
    bool recordEnable = true;
    bool _isLogedIn = false;

    bool _isWayland = false;
    inline bool isSupportWayland() const {
        return _isWayland;
    }
#ifdef WAYLAND
    std::unique_ptr<WaylandCapture> _waylandCapture;
#endif

    bool _bVideoFilp = false;
};

#endif // NEMOSDKADAPTOR_H
