﻿#include "OpenGLDisplay.h"

#include <QOpenGLShader>
#include <QOpenGLTexture>
#include <QCoreApplication>
#include <QResizeEvent>

#define PROGRAM_VERTEX_ATTRIBUTE 0
#define PROGRAM_TEXCOORD_ATTRIBUTE 1

#define ATTRIB_VERTEX 0
#define ATTRIB_TEXTURE 1

#define VERTEXIN 0
#define TEXTUREIN 1

static const char kVertexSource[] =
        "attribute vec4 vertexIn; \n"
        "attribute vec2 textureIn; \n"
        "varying vec2 textureOut;  \n"
        "void main(void)           \n"
        "{                         \n"
        "gl_Position = vertexIn; \n"
        "textureOut = textureIn; \n"
        "}";

static const char kFragmentSouece[] = "varying highp vec2 textureOut; \
    uniform sampler2D tex_y; \
    uniform sampler2D tex_u; \
    uniform sampler2D tex_v; \
    void main(void) \
    { \
        highp vec3 yuv; \
        highp vec3 rgb; \
        yuv.x = texture2D(tex_y, textureOut).r; \
        yuv.y = texture2D(tex_u, textureOut).r - 0.5; \
        yuv.z = texture2D(tex_v, textureOut).r - 0.5; \
        rgb = mat3( 1,       1,         1, \
                    0,       -0.39465,  2.03211, \
                    1.13983, -0.58060,  0) * yuv; \
        gl_FragColor = vec4(rgb, 1); \
    }";

struct OpenGLDisplay::OpenGLDisplayImpl
{
    OpenGLDisplayImpl()
        : mBufYuv(nullptr)
        , mFrameSize(0)
    {}

    std::shared_ptr<uint8_t> mBufYuv;
    int                     mFrameSize;
    int                     mRotation;
    int                     mSarWidth;
    int                     mSarHeight;
    bool                    mFlip;

    QOpenGLShader*          mVShader;
    QOpenGLShader*          mFShader;
    QOpenGLShaderProgram*   mShaderProgram;

    QOpenGLTexture*         mTextureY;
    QOpenGLTexture*         mTextureU;
    QOpenGLTexture*         mTextureV;

    GLuint                  id_y, id_u, id_v;
    int                     textureUniformY, textureUniformU, textureUniformV;
    GLsizei                 mVideoW, mVideoH;

};

const GLfloat OpenGLDisplay::_defaultVertices[] =
{
    -1.0f, -1.0f,
    1.0f,   -1.0f,
    -1.0f,  1.0f,
    1.0f,    1.0f,
};

const GLfloat OpenGLDisplay::_defaultCoord[] =
{
    0.0f,  1.0f,
    1.0f,  1.0f,
    0.0f,  0.0f,
    1.0f,  0.0f,
};

const GLfloat OpenGLDisplay::_defaultCoord_mirror[] =
{
    1.0f,  1.0f,
    0.0f,  1.0f,
    1.0f,  0.0f,
    0.0f,  0.0f,
};

const GLfloat OpenGLDisplay::_coord_90[] =
{
    1.0f,  1.0f,
    1.0f,  0.0f,
    0.0f,  1.0f,
    0.0f,  0.0f,
};

const GLfloat OpenGLDisplay::_coord_90_mirror[] =
{
    1.0f,  0.0f,
    1.0f,  1.0f,
    0.0f,  0.0f,
    0.0f,  1.0f,
};

const GLfloat OpenGLDisplay::_coord_180[] =
{
    1.0f,  0.0f,
    0.0f,  0.0f,
    1.0f,  1.0f,
    0.0f,  1.0f,
};

const GLfloat OpenGLDisplay::_coord_180_mirror[] =
{
    0.0f,  0.0f,
    1.0f,  0.0f,
    0.0f,  1.0f,
    1.0f,  1.0f,
};

const GLfloat OpenGLDisplay::_coord_270[] =
{
    0.0f,  0.0f,
    0.0f,  1.0f,
    1.0f,  0.0f,
    1.0f,  1.0f,
};

const GLfloat OpenGLDisplay::_coord_270_mirror[] =
{
    0.0f,  1.0f,
    0.0f,  0.0f,
    1.0f,  1.0f,
    1.0f,  0.0f,
};

OpenGLDisplay::OpenGLDisplay(QWidget* parent)
    : QOpenGLWidget(parent),
      impl(new OpenGLDisplayImpl),
      _viewWidth(1),
      _viewHeight(1)
{
    memcpy(_vertices, _defaultVertices, sizeof(_defaultVertices));
    memcpy(_coord, _defaultCoord, sizeof(_defaultCoord));
}

OpenGLDisplay::~OpenGLDisplay()
{
    vbo.destroy();
}

void OpenGLDisplay::initBuffer(int bsize)
{
    Q_UNUSED(bsize);
}

void OpenGLDisplay::requestRender(const std::shared_ptr<uint8_t>& data, int length, int width, int height, int sarWidth, int sarHeight, int rotation, bool flip)
{
    Q_UNUSED(length);

    impl->mVideoW = width;
    impl->mVideoH = height;
    impl->mSarWidth = sarWidth;
    impl->mSarHeight = sarHeight;
    impl->mRotation = rotation;
    impl->mFlip = flip;

    impl->mBufYuv = data;

    update();
}

void OpenGLDisplay::clearDraw()
{
    impl->mBufYuv.reset();
    impl->mBufYuv = nullptr;
}

void OpenGLDisplay::initializeGL()
{
    initializeOpenGLFunctions();

    impl->mVShader = new QOpenGLShader(QOpenGLShader::Vertex, this);

    bool ret =impl->mVShader->compileSourceCode(kVertexSource);
    qDebug("vshader:%d",ret);
    impl->mFShader = new QOpenGLShader(QOpenGLShader::Fragment,this);

    ret = impl->mFShader->compileSourceCode(kFragmentSouece);
    qDebug("fshader:%d",ret);

    impl->mShaderProgram = new QOpenGLShaderProgram(this);
    impl->mShaderProgram->addShader(impl->mVShader);
    impl->mShaderProgram->addShader(impl->mFShader);
    impl->mShaderProgram->bindAttributeLocation("vertexIn", VERTEXIN);
    impl->mShaderProgram->bindAttributeLocation("textureIn", TEXTUREIN);
    impl->mShaderProgram->link();
    impl->mShaderProgram->bind();

    impl->mShaderProgram->enableAttributeArray(VERTEXIN);
    impl->mShaderProgram->enableAttributeArray(TEXTUREIN);
    impl->mShaderProgram->setAttributeBuffer(VERTEXIN, GL_FLOAT, 0, 2, 2 * sizeof(GLfloat));
    impl->mShaderProgram->setAttributeBuffer(TEXTUREIN, GL_FLOAT, 8 * sizeof(GLfloat), 2, 2 * sizeof(GLfloat));

    impl->textureUniformY = impl->mShaderProgram->uniformLocation("tex_y");
    impl->textureUniformU = impl->mShaderProgram->uniformLocation("tex_u");
    impl->textureUniformV = impl->mShaderProgram->uniformLocation("tex_v");
    impl->mTextureY = new QOpenGLTexture(QOpenGLTexture::Target2D);
    impl->mTextureU = new QOpenGLTexture(QOpenGLTexture::Target2D);
    impl->mTextureV = new QOpenGLTexture(QOpenGLTexture::Target2D);
    impl->mTextureY->create();
    impl->mTextureU->create();
    impl->mTextureV->create();
    impl->id_y = impl->mTextureY->textureId();
    impl->id_u = impl->mTextureU->textureId();
    impl->id_v = impl->mTextureV->textureId();
    glActiveTexture(GL_TEXTURE0);  //激活纹理单元GL_TEXTURE0,系统里面的
    glBindTexture(GL_TEXTURE_2D, impl->id_y);

    glActiveTexture(GL_TEXTURE1); //激活纹理单元GL_TEXTURE1
    glBindTexture(GL_TEXTURE_2D, impl->id_u);
    glActiveTexture(GL_TEXTURE2); //激活纹理单元GL_TEXTURE2
    glBindTexture(GL_TEXTURE_2D, impl->id_v);

    glClearColor (0.0, 0.0, 0.0, 1.0); // set the background color
}

void OpenGLDisplay::resizeGL(int w, int h)
{
    if (h == 0) { // prevents being divided by zero
        h = 1;    // set the height to 1
    }

    _viewWidth = w;
    _viewHeight = h;
    // Set the viewport
    glViewport(0, 0, w, h);
    update();
}

void OpenGLDisplay::paintGL()
{
    if (impl->mBufYuv == nullptr) {
        return;
    }

    bool flip = impl->mFlip;
    unsigned int width = impl->mVideoW;
    unsigned int height = impl->mVideoH;
    unsigned int widthAfterRotation = 0;
    unsigned int heightAfterRotation = 0;

    float v_left = -1.0f;
    float v_top = 1.0f;
    float v_right = 1.0f;
    float v_bottom = -1.0f;

    // normal rotate degree
    switch (impl->mRotation) {
    case 0:
        memcpy(_coord, flip ? _defaultCoord_mirror : _defaultCoord, sizeof(_defaultCoord));
        widthAfterRotation = width;
        heightAfterRotation = height;
        break;
    case 1:
        memcpy(_coord, flip ? _coord_90_mirror : _coord_90, sizeof(_coord_90));
        widthAfterRotation = height;
        heightAfterRotation = width;
        break;
    case 2:
        memcpy(_coord, flip ? _coord_180_mirror : _coord_180, sizeof(_coord_180));
        widthAfterRotation = width;
        heightAfterRotation = height;
        break;
    case 3:
        memcpy(_coord, flip ? _coord_270_mirror : _coord_270, sizeof(_coord_270));
        widthAfterRotation = height;
        heightAfterRotation = width;
        break;
    }

    float frameAspect = 1.0 * widthAfterRotation / heightAfterRotation * impl->mSarWidth / impl->mSarHeight;
    float viewAspect = 1.0 * _viewWidth / _viewHeight;
    float ratio = frameAspect / viewAspect;

    if ((((frameAspect >= 1) && (viewAspect >= 1)) || ((frameAspect <= 1) && (viewAspect <= 1)))) {
        if (ratio > 1) {
            v_bottom = -1 / ratio;
            v_top = -v_bottom;
        } else {
            v_left = -ratio;
            v_right = ratio;
        }
    } else {
        if (ratio < 1) {
            v_left = -ratio;
            v_right = ratio;
        } else {
            v_bottom = -1 / ratio;
            v_top = -v_bottom;
        }
    }

    _vertices[0] = _vertices[4] = v_left;
    _vertices[1] = _vertices[3] = v_bottom;
    _vertices[2] = _vertices[6] = v_right;
    _vertices[5] = _vertices[7] = v_top;

    glVertexAttribPointer(ATTRIB_VERTEX, 2, GL_FLOAT, 0, 0, _vertices);
    glEnableVertexAttribArray(ATTRIB_VERTEX);
    glVertexAttribPointer(ATTRIB_TEXTURE, 2, GL_FLOAT, 0, 0, _coord);
    glEnableVertexAttribArray(ATTRIB_TEXTURE);

    glClearColor (0.0, 0.0, 0.0, 1.0); // set the background color
    glClear(GL_COLOR_BUFFER_BIT);

    glActiveTexture(GL_TEXTURE0);
    // Use the texture generated from y to generate texture
    glBindTexture(GL_TEXTURE_2D, impl->id_y);

    // Fixes abnormality with 174x100 yuv data
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
    glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
    glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);

    // Use the memory mBufYuv data to create a real y data texture
    glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, impl->mVideoW, impl->mVideoH, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, impl->mBufYuv.get());
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    // Load u data texture
    glActiveTexture(GL_TEXTURE1);//Activate texture unit GL_TEXTURE1
    glBindTexture(GL_TEXTURE_2D, impl->id_u);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, impl->mVideoW / 2, impl->mVideoH / 2
                 , 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, (char*)impl->mBufYuv.get() + impl->mVideoW * impl->mVideoH);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    // Load v data texture
    glActiveTexture(GL_TEXTURE2);//Activate texture unit GL_TEXTURE2
    glBindTexture(GL_TEXTURE_2D, impl->id_v);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, impl->mVideoW / 2, impl->mVideoH / 2
                 , 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, (char*)impl->mBufYuv.get() + impl->mVideoW * impl->mVideoH * 5 / 4);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    // Specify y texture to use the new value can only use 0, 1, 2, etc. to represent
    // the index of the texture unit, this is the place where opengl is not humanized
    //0 corresponds to the texture unit GL_TEXTURE0 1 corresponds to the
    // texture unit GL_TEXTURE1 2 corresponds to the texture unit GL_TEXTURE2
    glUniform1i(impl->textureUniformY, 0);
    // Specify the u texture to use the new value
    glUniform1i(impl->textureUniformU, 1);
    // Specify v texture to use the new value
    glUniform1i(impl->textureUniformV, 2);
    // Use the vertex array way to draw graphics
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
}

void OpenGLDisplay::showEvent(QShowEvent *event)
{
    if (nullptr == impl->mBufYuv) {
        qDebug() << __func__ << "clear cache view";
        glClearColor (0.0, 0.0, 0.0, 1.0);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        update();
    }
}
