﻿#include "yuvdisplaylabel.h"

#include <QPainter>
#include <QDebug>

#include "NemoSdkAdaptor.h"

YUVDisplayLabel::YUVDisplayLabel(QWidget *parent) :
    QLabel(parent),
    _bufferLen(0),
    _rgbBuffer(nullptr),
    _desWidth(0),
    _desHeight(0),
    _srcWidth(0),
    _srcHeight(0),
    _rotation(0)
{
    connect(this, &YUVDisplayLabel::sigRgbViewReady, this, &YUVDisplayLabel::onRgbViewReady);
}

YUVDisplayLabel::~YUVDisplayLabel()
{
    if (_rgbBuffer != nullptr) {
        free(_rgbBuffer);
        _rgbBuffer = nullptr;
    }
}

void YUVDisplayLabel::paintEvent(QPaintEvent *)
{
#if defined(__mips__)
	if (_yuvBuffer.get()) {
		QPainter painter(this);

		QImage image(_srcWidth, _srcHeight, QImage::Format_ARGB32);
		if (image.isNull()) return;

		for(int y = 0; y < _srcHeight; y++)
		{
			for(int x = 0; x < _srcWidth; x++)
			{
				int yIndex = y * _srcWidth + x;
				int uIndex = (y / 2) * (_srcWidth / 2) + x / 2 + _srcWidth * _srcHeight;
				int vIndex = (y / 2) * (_srcWidth / 2) + x / 2 + _srcWidth * _srcHeight * 5 / 4;

				int Y = _yuvBuffer.get()[yIndex];
				int U = _yuvBuffer.get()[uIndex] - 128;
				int V = _yuvBuffer.get()[vIndex] - 128;

				int R = Y + 1.402 * V;
				int G = Y - 0.34414 * U - 0.71414 * V;
				int B = Y + 1.772 * U;

				R = (R < 0) ? 0 : ((R > 255) ? 255 : R);
				G = (G < 0) ? 0 : ((G > 255) ? 255 : G);
				B = (B < 0) ? 0 : ((B > 255) ? 255 : B);

				QRgb color = qRgb(R, G, B);
				image.setPixel(x, y, color);
			}
		}

		int targetWidth = this->width();

		// TODO: there can scale once
		if (_rotation == 1 || _rotation == 3) {
			QImage img = image.scaled(QSize(this->height() * 9 / 16, this->height()), Qt::KeepAspectRatio, Qt::FastTransformation);
			targetWidth = img.rect().width();
		} else {
			QImage img = image.scaled(this->size(), Qt::KeepAspectRatio, Qt::FastTransformation);
			targetWidth = img.rect().width();
		}

		int left = (this->width() - targetWidth) / 2;
		QRectF target(left, 0, targetWidth , this->height());
		QRectF source(0, 0, _desWidth, _desHeight);
		painter.drawImage(target, image, source);
	}
#else
	if (_rgbBuffer) {
		QPainter painter(this);
		QImage tmpImg(static_cast<uchar *>(_rgbBuffer), _desWidth, _desHeight, QImage::Format_RGB32);
		if (tmpImg.isNull()) return;

		int targetWidth = this->width();

		// TODO: there can scale once
		if (_rotation == 1 || _rotation == 3) {
			QImage img = tmpImg.scaled(QSize(this->height() * 9 / 16, this->height()), Qt::KeepAspectRatio, Qt::FastTransformation);
			targetWidth = img.rect().width();
		} else {
			QImage img = tmpImg.scaled(this->size(), Qt::KeepAspectRatio, Qt::FastTransformation);
			targetWidth = img.rect().width();
		}
		int left = (this->width() - targetWidth) / 2;
		QRectF target(left, 0, targetWidth , this->height());
		QRectF source(0, 0, _desWidth, _desHeight);
		painter.drawImage(target, tmpImg, source);
	}
#endif
}

void YUVDisplayLabel::requestRender(const std::shared_ptr<uint8_t>& data, size_t length, int width, int height, int sarWidth, int sarHeight, int rotation, bool flip)
{
    _rotation = rotation;
    _srcWidth = width;
    _srcHeight = height;
    _desWidth =  rotation == 1 || rotation == 3 ? height : width;
    _desHeight = rotation == 1 || rotation == 3 ? width : height;

#if defined(__mips__)
	_yuvBuffer = data;
#else
	if (_bufferLen < width * height)
	{
		_bufferLen = width * height;
		if (_rgbBuffer != nullptr) {
			free(_rgbBuffer);
			_rgbBuffer = nullptr;
		}
		_rgbBuffer = (uint8_t*)malloc(width * height * 4);
	}
	NemoSdkAdaptor::getInstance()->convertYUV2ARGB(data.get(), _rgbBuffer, length, width, height, rotation, flip);
#endif
    emit sigRgbViewReady();
}

void YUVDisplayLabel::onRgbViewReady()
{
    update();
}
